/**
 * This module defines the user interface.
 */
import { Feature as Graphic, Geometry } from "arcgis-rest-api";
/**
 * This interface is for objects that may or may not have a toJson function.
 * Intended for use as an Intersection Type along with arcgis-rest-api Geometry
 * For instances that may be either a basic object representing a geometry or
 * a specialized ArcGIS API Geometry object.
 */
export interface IMayHaveToJson {
    /**
     * ArcGIS API object will have a toJson function that converts it to a standard JavaScript object.
     */
    toJson?: () => any;
}
/**
 * The *detail* of a *buffer* CustomEvent.
 */
export interface IBufferEventDetail {
    /**
     * The geometry or geometries to be buffered.
     */
    geometry: Geometry | Geometry[] | null;
    /**
     * The distance(s) around each geometry to buffer.
     */
    distance: number | number[] | null;
    /**
     * Measurement unit.
     * @see https://developers.arcgis.com/javascript/jsapi/bufferparameters-amd.html#unit
     */
    unit: number;
    /**
     * Boolean value indicating that the output buffer geometries should be unioned.
     */
    unionResults: boolean;
}
/**
 * UI for the Buffer operation on an ArcGIS Server Geometry service.
 * @class
 */
export default class BufferUI {
    root: HTMLElement;
    /**
     * The HTML form that the user interacts with.
     *
     * This form emits the following CustomEvents.
     *
     * * buffer - This event is emitted when a user submits the form with valid values. The detail property of this event is an IBufferEventDetail.
     * * clear-graphics - This event is emitted when the user clicks the "Clear Graphics" button.
     */
    form: HTMLFormElement;
    /**
     * Creates a new instance of BufferUI
     * @param root The HTML element that will contain the HTML form of this control.
     */
    constructor(root: HTMLElement);
    /**
     * Gets the distances entered in the distances box.
     */
    readonly distances: number | number[] | null;
    /**
     * Adds a feature to the UI's list.
     * @param feature A feature
     */
    addFeature(feature: Graphic): void;
    /**
     * Gets all of the geometries stored in the list items' data-geometry attributes
     */
    readonly geometries: Geometry | Geometry[] | null;
    /**
     * Returns an integer representing a measurement unit.
     */
    readonly unit: number;
    /**
     * Returns a boolean value indicating if the union checkbox is currently checked.
     */
    readonly unionResults: boolean;
    /**
     * Clears the list of geometries in the UI.
     */
    clearGeometryList(): void;
}
