"use strict";
/**
 * This module defines the user interface.
 */
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var units_1 = require("./units");
/**
 * The distance input can have either a single number or a comma separated
 * list of numbers. This Regex will match valid values for that control.
 */
var distancesPattern = /\d+(?:\.\d+)?([,\s]+\d+(?:\.\d+)?)*/;
/**
 * The HTML markup for the input form.
 */
var template = "<form class=\"buffer-ui\">\n<div>\n\t<label>Distances</label>\n  <input name=\"distances\" placeholder=\"e.g 200,300\" pattern=\"" + distancesPattern.source + "\" title=\"Must be a number or list of numbers.\" required=\"required\" />\n</div>\n<div>\n\t<label>Measurement Unit</label>\n\t<select name=\"unit\"></select>\n</div>\n<fieldset>\n\t<label>Geometries</label>\n\t<ul class=\"geometry-list\"></ul>\n\t<button class=\"clear-geometries\" type=\"button\">Clear Geometries</button>\n</fieldset>\n<fieldset>\n\t<ul class=\"buffer-options-list\">\n\t\t<li><label><input type=\"checkbox\" name=\"union\" />Union results</label></li>\n\t</ul>\n</fieldset>\n<div>\n\t<button type=\"submit\">Buffer</button>\n</div>\n<div>\n\t<label>Results Layer</label>\n\t<button type=\"button\" class=\"clear-result-graphics-button\">Clear result graphics</button>\n</div>\n</form>";
/**
 * Converts the template HTML markup string into an HTML DOM element,
 * then clones its form.
 * @param {string} templateMarkup - HTML markup string.
 * @returns {HTMLFormElement}
 */
function getFormFromTemplate(templateMarkup) {
    var form = document.createElement("form");
    form.innerHTML = template;
    var unitSelect = form.unit;
    unitSelect.appendChild(units_1.createUnitSelectContents("Foot"));
    return form;
}
/**
 * If given a graphic object, returns its geometry property.
 * If given a geometry object, simply returns the input object.
 * @param featureOrGeometry A graphic or geometry object.
 * @throws {TypeError} Thrown if the input object is neither a geometry nor contains a geometry.
 */
function getGeometry(featureOrGeometry) {
    if (featureOrGeometry.geometry) {
        return featureOrGeometry.geometry;
    }
    else if (featureOrGeometry.hasOwnProperty("x") ||
        featureOrGeometry.hasOwnProperty("points") ||
        featureOrGeometry.hasOwnProperty("rings") ||
        featureOrGeometry.hasOwnProperty("paths")) {
        return featureOrGeometry;
    }
    throw new TypeError("Input must be a Graphic or Geometry");
}
/**
 * UI for the Buffer operation on an ArcGIS Server Geometry service.
 * @class
 */
var BufferUI = /** @class */ (function () {
    /**
     * Creates a new instance of BufferUI
     * @param root The HTML element that will contain the HTML form of this control.
     */
    function BufferUI(root) {
        this.root = root;
        var self = this;
        var form = getFormFromTemplate(template);
        this.root.appendChild(form);
        this.form = form;
        form.onsubmit = function () {
            var geometries = { self: self };
            if (geometries) {
                var evt = new CustomEvent("buffer", {
                    detail: {
                        geometry: self.geometries,
                        distance: self.distances,
                        unit: self.unit,
                        unionResults: self.unionResults
                    }
                });
                form.dispatchEvent(evt);
            }
            return false;
        };
        var clearGeometriesButton = this.root.querySelector("button.clear-geometries");
        clearGeometriesButton.onclick = function () {
            self.clearGeometryList();
        };
        var clearGraphicsButton = this.root.querySelector("button.clear-result-graphics-button");
        clearGraphicsButton.onclick = function () {
            var evt = new CustomEvent("clear-graphics");
            form.dispatchEvent(evt);
        };
        this.form = form;
        this.root.appendChild(form);
    }
    Object.defineProperty(BufferUI.prototype, "distances", {
        /**
         * Gets the distances entered in the distances box.
         */
        get: function () {
            var distances = null;
            var s = this.form.distances.value;
            if (s) {
                distances = s.split(/[,\s]+/).map(function (st) {
                    return parseFloat(st);
                });
            }
            if (distances && distances.length === 1) {
                return distances[0];
            }
            return distances;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Adds a feature to the UI's list.
     * @param feature A feature
     */
    BufferUI.prototype.addFeature = function (feature) {
        var list = this.root.querySelector(".geometry-list");
        var li = document.createElement("li");
        var geometry = getGeometry(feature);
        if (geometry.toJson) {
            geometry = geometry.toJson();
        }
        li.dataset.geometry = JSON.stringify(geometry);
        // TODO: If input is a feature, make the list item's text content more descriptive of the feature using its properties.
        li.textContent = "Geometry";
        list.appendChild(li);
    };
    Object.defineProperty(BufferUI.prototype, "geometries", {
        /**
         * Gets all of the geometries stored in the list items' data-geometry attributes
         */
        get: function () {
            var geometries = new Array();
            var listItems = this.root.querySelectorAll(".geometry-list > li");
            try {
                for (var _a = __values(listItems.values()), _b = _a.next(); !_b.done; _b = _a.next()) {
                    var li = _b.value;
                    var gJson = li.dataset.geometry;
                    var g = JSON.parse(gJson);
                    geometries.push(g);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                }
                finally { if (e_1) throw e_1.error; }
            }
            if (geometries.length < 1) {
                return null;
            }
            else if (geometries.length === 1) {
                return geometries[0];
            }
            return geometries;
            var e_1, _c;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BufferUI.prototype, "unit", {
        /**
         * Returns an integer representing a measurement unit.
         */
        get: function () {
            return parseInt(this.form.unit.value, 10);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BufferUI.prototype, "unionResults", {
        /**
         * Returns a boolean value indicating if the union checkbox is currently checked.
         */
        get: function () {
            return Boolean(this.form.querySelector("[name=union]:checked"));
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Clears the list of geometries in the UI.
     */
    BufferUI.prototype.clearGeometryList = function () {
        var ul = this.root.querySelector(".geometry-list");
        ul.innerHTML = "";
    };
    return BufferUI;
}());
exports.default = BufferUI;
